@@ -5,7 +5,7 @@
 #  open source, and has the attribution requirements (GPL Section 7) at
 #  https://statnet.org/attribution .
 #
-#  Copyright 2003-2021 Statnet Commons
+#  Copyright 2003-2022 Statnet Commons
 ################################################################################
 ###############################################################################
 # The <ergm> function fits ergms from a specified formula returning either
@@ -127,7 +127,7 @@
 #'   \code{\link[network]{network}} object or a matrix that can be
 #'   coerced to a \code{\link[network]{network}} object.  For the
 #'   details on the possible \code{<model terms>}, see
-#'   \code{\link{ergm-terms}} and Morris, Handcock and Hunter (2008)
+#'   \code{\link{ergmTerm}} and Morris, Handcock and Hunter (2008)
 #'   for binary ERGM terms and Krivitsky (2012) for valued ERGM terms
 #'   (terms for weighted edges).  To create a
 #'   \code{\link[network]{network}} object in \R, use the
@@ -145,7 +145,7 @@
 #' using syntax similar to the \code{formula} argument, on the
 #' right-hand side. Multiple constraints
 #' may be given, separated by \dQuote{+} and \dQuote{-} operators. (See
-#' [ERGM constraints][ergm-constraints] for the explanation of
+#' [ERGM constraints][ergmConstraint] for the explanation of
 #' their semantics.)
 #' Together with the model terms in the formula and the reference measure, the constraints
 #' define the distribution of networks being modeled.
@@ -159,7 +159,7 @@
 #' 
 #' The default is \code{~.}, for an unconstrained model.
 #' 
-#' See the [ERGM constraints][ergm-constraints] documentation for
+#' See the [ERGM constraints][ergmConstraint] documentation for
 #' the constraints implemented in the **[ergm][ergm-package]**
 #' package. Other packages may add their own constraints.
 #' 
@@ -189,7 +189,7 @@
 #'   arguments to the proposal should be specified through the
 #'   \code{obs.prop.args} argument to \code{\link{control.ergm}}.
 #' 
-#'   See the [ERGM constraints][ergm-constraints] documentation for
+#'   See the [ERGM constraints][ergmConstraint] documentation for
 #'   the constraints implemented in the **[ergm][ergm-package]**
 #'   package. Other packages may add their own constraints.
 #'     
@@ -264,7 +264,7 @@
 #' \item{network}{Network passed on the left-hand side of `formula`. If `target.stats` are passed, it is replaced by the network returned by [san()].}
 #' \item{newnetworks}{A list of the final networks at the end of the MCMC
 #' simulation, one for each thread.}
-#' \item{newnetwork}{The first (possibly only) element of \code{netwonetworks}.}
+#' \item{newnetwork}{The first (possibly only) element of \code{newnetworks}.}
 #' \item{coef.init}{The initial value of \eqn{\theta}.}
 #' \item{est.cov}{The covariance matrix of the model statistics in the final MCMC sample.}
 #' \item{coef.hist, steplen.hist, stats.hist, stats.obs.hist}{
@@ -349,7 +349,7 @@
 #' Prototype Packages for Managing and Animating Longitudinal
 #' Network Data: \pkg{dynamicnetwork} and \pkg{rSoNIA}.
 #' \emph{Journal of Statistical Software}, 24(7).
-#' \url{https://www.jstatsoft.org/v24/i07/}.
+#' \doi{10.18637/jss.v024.i07}
 #' 
 #' 
 #' Butts CT (2007).
@@ -359,7 +359,7 @@
 #' Butts CT (2008).
 #' \pkg{network}: A Package for Managing Relational Data in \R.
 #' \emph{Journal of Statistical Software}, 24(2).
-#' \url{https://www.jstatsoft.org/v24/i02/}.
+#' \doi{10.18637/jss.v024.i02}
 #' 
 #' Butts C (2015).
 #' \pkg{network}: The Statnet Project (https://statnet.org). R package version 1.12.0, \url{https://cran.r-project.org/package=network}.
@@ -367,7 +367,7 @@
 #' Goodreau SM, Handcock MS, Hunter DR, Butts CT, Morris M (2008a).
 #' A \pkg{statnet} Tutorial.
 #' \emph{Journal of Statistical Software}, 24(8).
-#' \url{https://www.jstatsoft.org/v24/i08/}.
+#' \doi{10.18637/jss.v024.i08}
 #' 
 #' Goodreau SM, Kitts J, Morris M (2008b).
 #' Birds of a Feather, or Friend of a Friend? Using Exponential
@@ -409,7 +409,7 @@
 #' \pkg{ergm}: A Package to Fit, Simulate and Diagnose
 #' Exponential-Family Models for Networks.
 #' \emph{Journal of Statistical Software}, 24(3).
-#' \url{https://www.jstatsoft.org/v24/i03/}.
+#' \doi{10.18637/jss.v024.i03}
 #'
 #' Karwa V, Krivitsky PN, and Slavkovi\'{c} AB (2017). Sharing Social Network
 #' Data: Differentially Private Estimation of Exponential-Family Random
@@ -424,7 +424,7 @@
 #' Specification of Exponential-Family Random Graph Models:
 #' Terms and Computational Aspects.
 #' \emph{Journal of Statistical Software}, 24(4).
-#' \url{https://www.jstatsoft.org/v24/i04/}.
+#' \doi{10.18637/jss.v024.i04}
 #' 
 #' Snijders, T.A.B. (2002),
 #' Markov Chain Monte Carlo Estimation of Exponential Random Graph Models.
@@ -432,7 +432,7 @@
 #' Available from 
 #' \url{https://www.cmu.edu/joss/content/articles/volume3/Snijders.pdf}.
 #' 
-#' @seealso [`network`], [`%v%`], [`%n%`], [ergm-terms], [`ergmMPLE`],
+#' @seealso [`network`], [`%v%`], [`%n%`], [`ergmTerm`], [`ergmMPLE`],
 #' [summary.ergm()]
 #' 
 #' @examples
@@ -788,6 +788,8 @@ ergm <- function(formula, response=NULL,
                                                                nonvar_action = control$MPLE.nonvar))
          )
 
+  initialfit$xmat.full <- NULL # No longer needed but takes up space.
+
   estimate.desc <- switch(estimate,
                           MPLE = if(MPLE.is.MLE) "Maximum Likelihood"
                                  else "Maximum Pseudolikelihood",
